<?php

namespace Ulmus\User\Authorize\Header;

use Psr\Http\Message\ServerRequestInterface;
use Ulmus\User\Lib\AuthenticationMethodEnum;

class BasicMethod implements MethodInterface
{
    public function __construct(
        protected string|array $arguments
    ) {}

    public function execute(ServerRequestInterface $request) : ServerRequestInterface
    {
        if ( false === $decoded = base64_decode($this->arguments) ) {
            throw new \RuntimeException("Base64 decoding of given username:password failed");
        }

        list($username, $password) = explode(':', $decoded) + [ null, null ];

        if ( empty($username) ) {
            throw new \RuntimeException("A username must be provided");
        }
        elseif ( empty($password) ) {
            throw new \RuntimeException("A password must be provided");
        }

        return $request->withAttribute('authentication_middleware:method', AuthenticationMethodEnum::UsernamePassword)
            ->withAttribute('authentication_middleware:username', $username)
            ->withAttribute('authentication_middleware:password', $password);
    }
}