<?php

namespace Ulmus\User\Authorize\Header;

use Psr\Http\Message\ServerRequestInterface;
use Ulmus\User\Authorize\Bearer\JsonWebTokenDecoder;
use Ulmus\User\Entity\UserInterface;
use Ulmus\User\Lib\Authenticate;

class BearerMethod implements MethodInterface
{
    protected JsonWebTokenDecoder $jwt;

    public function __construct(
        # protected Authenticate $authorize,
        # protected UserInterface $user,
        protected string $token
    ) {}

    public function execute(ServerRequestInterface $request) : ServerRequestInterface
    {
        # $this->authorize->user = $this->user;

        switch($this->autodetectTokenType()) {
            case BearerTokenTypeEnum::JsonWebToken:
                $payload = $this->jwt->getPayload();

                if ( $payload['sub'] ?? false) {
                    $request = $request->withAttribute('authentication_middleware:user_id', $payload['sub']);
                }
                else {
                    throw new \InvalidArgumentException("Given JsonWebToken is missing a 'sub' key (which concords to user id)");
                }

                break;

            case BearerTokenTypeEnum::UniqueKey:
                # @TODO
                break;
        }

        return $request;
    }

    public function autodetectTokenType() : BearerTokenTypeEnum
    {
        $this->jwt = new JsonWebTokenDecoder($this->token, getenv('LEAN_RANDOM'));

        if ( $this->jwt->isJWT() ) {
            return BearerTokenTypeEnum::JsonWebToken;
        }

        return BearerTokenTypeEnum::UniqueKey;
    }
}