<?php

namespace Ulmus\User\Authorize;

use Psr\Http\Message\ServerRequestInterface;
use Ulmus\User\Common\AuthorizeContentTypeEnum;
use Ulmus\User\Entity\{ DigestAuthUserInterface, UserInterface };
use Ulmus\User\Lib\Authenticate;
use Ulmus\User\Lib\HeaderAuthenticationTypeEnum;

class HeaderAuthentication implements AuthorizeMethodInterface
{
    public function __construct(
        protected Authenticate $authorize,
    ) { }

    public function connect(ServerRequestInterface $request): ServerRequestInterface
    {
        $type = HeaderAuthenticationTypeEnum::fromRequest($request);

        if ( null !== $type ) {
            list(, $value) = explode(' ', $request->getHeaderLine('Authorization'), 2) + [ null, null ];

            switch($type) {
                case HeaderAuthenticationTypeEnum::Basic:
                    $methodObj = new Header\BasicMethod($value);
                break;

                case HeaderAuthenticationTypeEnum::Digest:
                    #if (! $user instanceof DigestAuthUserInterface) {
                    #    throw new \RuntimeException("Your user entity must provide a valid hash of `user:realm:password` ");
                    #}

                    $methodObj = new Header\DigestMethod($value);
                break;

                case HeaderAuthenticationTypeEnum::Bearer:
                    $methodObj = new Header\BearerMethod($value);
                break;

                case HeaderAuthenticationTypeEnum::Token:
                    // @TODO !
                break;
            }
        }

        if (isset($methodObj)) {
            $request = $methodObj->execute($request);
        }

        return $request;
    }

    public function catchRequest(ServerRequestInterface $request) : bool
    {
        # Previously only catched from JSON content-type ; now accepts every connections

        /*foreach(array_merge($request->getHeader('Accept'), $request->getHeader('Content-Type')) as $accept) {
            foreach(explode(',', $accept) as $contentType) {
                if ( AuthorizeContentTypeEnum::tryFrom(strtolower($contentType)) ) {
                    return true;
                }
            }
        }*/

        return $request->hasHeader('authorization');
    }
}