<?php

namespace Ulmus\User\Authorize;

use Psr\Http\Message\ServerRequestInterface;
use Ulmus\User\Entity\UserInterface;
use Ulmus\User\Lib\Authenticate;
use Ulmus\User\Lib\AuthenticationMethodEnum;

class PostRequestAuthentication implements AuthorizeMethodInterface
{
    public function __construct(
        protected string $fieldUser = "email",
        protected string $postFieldUser = "email",
        protected string $postFieldPassword = "password",
    ) {}

    public function connect(ServerRequestInterface $request) : ServerRequestInterface
    {
        $post = $request->getParsedBody();

        return $request->withAttribute('authentication_middleware:method', AuthenticationMethodEnum::UsernamePassword)
            ->withAttribute('authentication_middleware:username', [ $this->fieldUser => $post[$this->postFieldUser] ])
            ->withAttribute('authentication_middleware:password', $post[$this->postFieldPassword]);
    }

    public function catchRequest(ServerRequestInterface $request): bool
    {
        $post = $request->getParsedBody();

        return strtoupper( $request->getMethod() ) === "POST" && isset($post[$this->postFieldUser], $post[$this->postFieldPassword]);
    }
}